import { Component, OnInit } from '@angular/core';
import { UntypedFormBuilder, UntypedFormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { NbDialogService } from '@nebular/theme';
import { User } from 'app/core/models/user';
import { AuthService } from 'app/core/services/auth.service';
import { FormComponent } from 'app/shared/components/base/form.component';
import { DialogConfirmationComponent } from 'app/shared/dialogs/dialog-confirmation/dialog-confirmation.component';
import { BackendError } from 'app/shared/models/backend-error';
import { NotificationService } from 'app/shared/services/notification.service';
import { combineLatest, forkJoin, Subject } from 'rxjs';
import { debounceTime, takeUntil, flatMap, filter } from 'rxjs/operators';
import { UpdateCourseDto } from '../../dto/update-course.dto';
import { Course } from '../../models/course';
import { CourseVisibility } from '../../models/course-visibility';
import { CourseService } from '../../services/course.service';
import { TinyMCEHelper } from 'app/shared/helper/tiny-mce.helper';
import { SubmitPolicy } from '../../models/submit-policy';
import { ErrorContainerConfig } from 'app/shared/components/error-container/error-container.component';
import { AppStateService } from 'app/core/services/app.state.service';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';
import { environment } from 'environments/environment';

@Component({
  selector: 'ngx-course-settings',
  templateUrl: './course-settings.component.html',
  styleUrls: ['./course-settings.component.scss']
})
export class CourseSettingsComponent extends FormComponent implements OnInit {
  courseManualUrl = environment.courseManualUrl;
  loading = true;
  course: Course;
  principal: SubatoPrincipal;
  courseForm: UntypedFormGroup;
  deletingCourse = false;
  reimportingCourse = false;
  CourseVisibility = CourseVisibility;
  SubmitPolicy = SubmitPolicy;
  error: BackendError;
  errorConfig: ErrorContainerConfig = {
    notFound: {
      message: 'Diesen Kurs konnten wir leider nicht finden.'
    }
  };
  
  constructor(
    private courseService: CourseService,
    private appStateService: AppStateService,
    private router: Router,
    private authService: AuthService,
    private dialogService: NbDialogService,
    private notificationService: NotificationService,
    private fb: UntypedFormBuilder,
    private route: ActivatedRoute) {
    super();
  }

  get visibilities(): CourseVisibility[] {
    return <any>Object.keys(CourseVisibility).filter(k => !isNaN(Number(k))).map(k => Number(k));
  }

  get submitPolicies(): SubmitPolicy[] {
    return <any>Object.keys(SubmitPolicy).filter(k => !isNaN(Number(k))).map(k => Number(k));
  }

  ngOnInit(): void {
    combineLatest(this.route.paramMap, this.authService.principal$)
      .pipe(
        takeUntil(this.unsubscribe),
        flatMap(([params, principal]) => {
          this.principal = principal;
          let courseId = Number(params.get("course_id"));
          return this.courseService.getById(courseId);
        })
      )
      .subscribe(course => {
        this.course = course;
        this.appStateService.setCourse(course);

        this.createForm(course);
        this.loading = false;
      }, error => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });

  }

  createForm(course?: Course) {
    this.courseForm = this.fb.group({
      name: [course?.name ?? '', [Validators.required]],
      shortName: [course?.shortName ?? '', [Validators.required]],
      description: [course?.description ?? ''],
      motd: [course?.motd ?? ''],
      showEmails: [course?.showEmails ?? false],
      visibility: [course?.visibility ?? null, [Validators.required]],
      submitPolicy: [course?.submitPolicy ?? null, [Validators.required]]
    });
  }

  save() {
    let dto: UpdateCourseDto = {
      name: this.courseForm.get('name').value,
      shortName: this.courseForm.get('shortName').value,
      description: this.courseForm.get('description').value,
      motd: this.courseForm.get('motd').value,
      showEmails: this.courseForm.get('showEmails').value,
      visibility: this.courseForm.get('visibility').value,
      submitPolicy: this.courseForm.get('submitPolicy').value,
    }

    this.courseService.update(this.course.id, dto)
      .pipe(takeUntil(this.unsubscribe))
      .subscribe((course) => {
        this.course = course;
        this.notificationService.showSuccess("Kurs erfolgreich aktualisiert");
        this.router.navigate(['/', 'course', course.id]);
      }, (error: any) => {
        if (BackendError.isBackendError(error)) {
          if (error.isValidation) {
            this.errors = this.getValidationErrors(error);
          }

          this.notificationService.showError(
            error.format(
              "Fehler bei Aktualisierung des Kurses",
            )
          );
        } else {
          throw error;
        }
      });
  }

  deleteCourse() {
    this.dialogService.open(DialogConfirmationComponent, {
      context: {
        title: `Löschung von Kurs: ${this.course.name}`,
        text: `Bist du wirklich sicher, dass du den Kurs '${this.course.name}' löschen willst? Alle verknüpften Übungen, Abgaben, Lösungen etc. werden damit permanent gelöscht.`,
        actionText: 'Löschen'
      }
    }).onClose.pipe(
      takeUntil(this.unsubscribe),
      filter(result => result && result.confirmed),
      flatMap(() => {
        this.deletingCourse = true;
        return this.courseService.delete(this.course.id);
      })
    ).subscribe(() => {
      this.deletingCourse = false;
      this.notificationService.showSuccess("Kurs erfolgreich gelöscht");
      this.router.navigate(['course']);
    }, error => {
      this.deletingCourse = false;

      if (BackendError.isBackendError(error)) {
        this.notificationService.showError(
          error.format(
            "Fehler beim Löschen des Kurses",
          )
        );
      } else {
        throw error;
      }
    })
  }

  reimport() {
    this.reimportingCourse = true;
    return this.courseService.reimport(this.course.id)
      .pipe(takeUntil(this.unsubscribe))
      .subscribe(course => {
        this.reimportingCourse = false;
        this.notificationService.showSuccess("Reimport erfolgreich");
      }, error => {
        this.reimportingCourse = false;

        if (BackendError.isBackendError(error)) {
          this.notificationService.showError(
            error.format(
              "Fehler bei Reimport",
            )
          );
        } else {
          throw error;
        }
      })
  }
}
