import { Component, OnInit } from '@angular/core';
import { AbstractControl, UntypedFormBuilder, UntypedFormGroup, ValidationErrors, ValidatorFn, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { ProgramModule } from 'app/features/modules/models/program-module';
import { FormComponent } from 'app/shared/components/base/form.component';
import { NotificationService } from 'app/shared/services/notification.service';
import { takeUntil, flatMap, debounceTime, filter } from 'rxjs/operators';
import { CreateCourseDto } from '../../dto/create-course.dto';
import { Course } from '../../models/course';
import { CourseService } from '../../services/course.service';
import { combineLatest, forkJoin, of } from 'rxjs';
import { AppStateService } from 'app/core/services/app.state.service';
import { Term } from 'app/features/modules/models/term';
import { BackendError } from 'app/shared/models/backend-error';
import { environment } from 'environments/environment';

export const templateFormValidator: ValidatorFn = (control: AbstractControl): ValidationErrors | null => {
  const useTemplate = control.get('useTemplate').value;
  const courseId = control.get('courseId').value;

  return useTemplate && courseId == null ? { courseMissing: true } : null;
};

@Component({
  selector: 'ngx-new-course',
  templateUrl: './new-course.component.html',
  styleUrls: ['./new-course.component.scss']
})
export class NewCourseComponent extends FormComponent implements OnInit {
  courseManualUrl = environment.courseManualUrl;

  term: Term;

  generalForm: UntypedFormGroup;
  templateForm: UntypedFormGroup;

  templateCourse: Course | null;
  selectedModules: ProgramModule[] = [];

  saving: boolean = false;

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private courseService: CourseService,
    private notificationService: NotificationService,
    private appStateService: AppStateService,
    private fb: UntypedFormBuilder) {
    super();
  }

  ngOnInit(): void {
    combineLatest(this.appStateService.term$).pipe(
      takeUntil(this.unsubscribe)
    )
    .subscribe(([term]) => {
      this.term = term;
      this.createForms();
    });
  }

  createForms() {
    this.generalForm = this.fb.group({
      name: ['', Validators.required],
      shortName: ['', Validators.required],
      permanent: [false, Validators.required],
    });

    this.templateForm = this.fb.group({
      useTemplate: [false],
      courseId: [null]
    }, { validators: templateFormValidator });
  }

  useTemplateChanged(val: boolean) {
    this.templateForm.patchValue({
      useTemplate: val
    });

    if (!this.useTemplate) {
      this.onCourseSelected(null);
    }
  }

  modulesChanged(modules: ProgramModule[]) {
    this.selectedModules = modules;
  }

  get useTemplate(): boolean {
    return this.templateForm.get('useTemplate').value;
  }

  onCourseSelected(course: Course) {
    this.templateForm.patchValue({
      courseId: course?.id
    });

    this.generalForm.patchValue({
      name: course?.name ?? '',
      shortName: course?.shortName ?? '',
    });

    this.templateCourse = course;
    this.selectedModules = course?.modules ?? [];
  }

  get permanent() {
    return this.generalForm.get('permanent').value;
  }

  save() {
    this.saving = true;

    let dto: CreateCourseDto = {
      name: this.generalForm.get('name').value,
      shortName: this.generalForm.get('shortName').value,
      permanent: this.generalForm.get('permanent').value,
      courseId: this.templateForm.get('courseId').value,
      moduleIds: this.selectedModules.map(m => m.id),
      termId: this.appStateService.getCurrentTerm()?.id
    };

    this.courseService.create(dto)
      .pipe(takeUntil(this.unsubscribe))
      .subscribe(course => {
        this.notificationService.showSuccess("Kurs erfolgreich erstellt");
        this.router.navigate(['course', course.id]);
        this.saving = false;
      }, error => {
        this.saving = false;

        if (BackendError.isBackendError(error)) {
          this.notificationService.showError(error.format(
            "Fehler bei Erstellung des Kurses",
          ));
        } else {
          throw error;
        }
      });
  }
}
