import { Component, OnInit, ViewChild } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { AuthService } from 'app/core/services/auth.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { NotificationService } from 'app/shared/services/notification.service';
import { combineLatest, forkJoin, Observable, of } from 'rxjs';
import { flatMap, map, takeUntil } from 'rxjs/operators';

import { TaskInstance } from 'app/features/exercises/models/task-instance';
import { Task } from 'app/features/tasks/models/task';
import { User } from 'app/core/models/user';
import { ExerciseService } from '../../services/exercise.service';
import { FeedbackComponent } from 'app/features/tasks/components/feedback/feedback.component';
import { NbPopoverDirective } from '@nebular/theme';
import { BackendError } from 'app/shared/models/backend-error';
import { ErrorContainerConfig } from 'app/shared/components/error-container/error-container.component';
import { TaskFilesComponent } from 'app/features/tasks/components/task-files/task-files.component';
import { Breadcrumb } from 'app/shared/components/breadcrumbs/breadcrumbs.component';
import { AppStateService } from 'app/core/services/app.state.service';
import { SubmissionService } from '../../services/submission.service';
import { Submission } from '../../models/submission';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';
import { Solution } from 'app/features/solutions/models/solution';
import { TaskService } from 'app/features/tasks/services/task.service';

@Component({
  selector: 'ngx-submit-solution',
  templateUrl: './submit-solution.component.html',
  styleUrls: ['./submit-solution.component.scss']
})
export class SubmitSolutionComponent extends BaseComponent implements OnInit {
  feedback = FeedbackComponent;
  @ViewChild(NbPopoverDirective) feedbackPopover: NbPopoverDirective;
  @ViewChild(TaskFilesComponent, { static: false }) taskFilesComponent!: TaskFilesComponent;

  taskInstance: TaskInstance;
  taskContent: string;
  submission: Submission;
  taskInstances: TaskInstance[] = [];
  loading: boolean = true;
  publishingSolution: boolean = false;
  prevTaskInstance: TaskInstance | null = null;
  nextTaskInstance: TaskInstance | null = null;

  principal: SubatoPrincipal;

  error: BackendError;
  errorConfig: ErrorContainerConfig = {
    notFound: {
      message: 'Diese Aufgabe konnten wir in diesem Übungsblatt leider nicht finden.'
    }
  };

  breadcrumbs: Breadcrumb[] = [];

  constructor(
    private exerciseService: ExerciseService,
    private submissionService: SubmissionService,
    private appStateService: AppStateService,
    private router: Router,
    private route: ActivatedRoute,
    private authService: AuthService,
    private notificationService: NotificationService) {
    super();
  }

  buildBreadcrumbs(taskInstance: TaskInstance) {
    this.breadcrumbs = [
      {
        id: 'exercises',
        params: {
          course_id: taskInstance.exercise.course.id
        },
      },
      {
        id: 'exercise',
        params: {
          exercise_id: taskInstance.exercise.id
        },
        ctx: taskInstance.exercise
      },
      {
        id: 'submitSolution',
        params: {
          exercise_id: taskInstance.exercise.id,
          task_id: taskInstance.task.id
        },
        ctx: taskInstance
      },
    ];
  }

  onTasksLoaded(taskInstances: TaskInstance[]) {
    this.taskInstances = taskInstances;
    this.updatePrevAndNext();
  }

  updatePrevAndNext() {
    this.prevTaskInstance = this.taskInstance.getPrevTaskInstance(this.taskInstances);
    this.nextTaskInstance = this.taskInstance.getNextTaskInstance(this.taskInstances);
  }

  onFeedbackSubmitted() {
    this.feedbackPopover.hide();
  }

  getTaskInstanceLink(taskInstance: TaskInstance) {
    return ['/', 'exercises', taskInstance.exercise.id, 'tasks', taskInstance.task.id];
  }

  ngOnInit(): void {
    combineLatest(this.route.paramMap, this.authService.principal$)
      .pipe(
        takeUntil(this.unsubscribe),
        flatMap(([params, principal]) => {
          this.loading = true;
          this.principal = principal;
          let exerciseId = Number(params.get("exercise_id"));
          let taskId = Number(params.get("task_id"));
          return forkJoin(
            this.exerciseService.getTaskInstance(exerciseId, taskId),
            this.loadSubmission(exerciseId)
          );
        }),
        flatMap(([taskInstance, submission]) => {
          this.taskInstance = taskInstance;
          this.submission = submission;

          return !taskInstance.task.archived
            ? this.exerciseService.getTaskContent(taskInstance.exercise.id, taskInstance.task.id)
            : of(null);
        })
      )
      .subscribe(content => {
        this.loading = false;
        this.taskContent = content;
        this.updatePrevAndNext();

        this.appStateService.setCourse(this.taskInstance.exercise.course);
        this.buildBreadcrumbs(this.taskInstance);
      }, (error: any) => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });
  }

  loadSubmission(exerciseId: number): Observable<Submission> {
    let loadSubmission = this.principal != null ?
      this.submissionService.getSubmissions({ exerciseId: exerciseId, userId: this.principal.id })
      : of(<Submission[]>[]);

    return loadSubmission.pipe(
      map(submissions => submissions[0]),
      flatMap(submission => submission ? this.submissionService.getById(submission.id) : of(null))
    )
  }

  selectTask(task: Task) {
    this.router.navigate(["exercises", this.taskInstance.exercise.id, "tasks", task.id]);
  }

  nextTask() {
    let ti = this.nextTaskInstance;
    if (ti) {
      this.selectTask(ti.task);
    }
  }

  prevTask() {
    let ti = this.prevTaskInstance;
    if (ti) {
      this.selectTask(ti.task);
    }
  }


  solutionAccent(solution: Solution) {
    if (solution.isGood) {
      return 'success';
    }
    else if (solution.isOk) {
      return 'warning';
    }
    else if (solution.isBad) {
      return 'danger';
    }
    else {
      return 'primary';
    }
  }

  publishSolution(publish: boolean) {
    this.publishingSolution = true;

    this.exerciseService.publishSolution(this.taskInstance.exercise.id, this.taskInstance.task.id, publish)
      .pipe(
        takeUntil(this.unsubscribe)
      ).subscribe(() => {
        this.publishingSolution = false;
        this.taskInstance.solutionPublic = publish;
        this.taskFilesComponent.refresh();
        this.notificationService.showSuccess(publish ? "Musterlösung erfolgreich veröffentlicht" : "Musterlösungen sind nun nicht mehr öffentlich");
      }, error => {
        this.publishingSolution = false;
        if (BackendError.isBackendError(error)) {
          this.notificationService.showError(
            error.format(
              "Fehler bei Veröffentlichung der Musterlösung",
            )
          );
        } else {
          throw error;
        }
      })
  }
}
