import { Component, Input, OnInit } from '@angular/core';
import { UntypedFormBuilder, UntypedFormGroup, Validators } from '@angular/forms';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { BackendError } from 'app/shared/models/backend-error';
import { NotificationService } from 'app/shared/services/notification.service';
import { takeUntil } from 'rxjs/operators';
import { CreateSolutionCommentDto } from '../../dto/create-solution-comment.dto';
import { SolutionCommentService } from '../../services/solution-comment.service';
import { SolutionComment } from '../../models/solution-comment';
import { AuthService } from 'app/core/services/auth.service';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';
import { Solution } from '../../models/solution';

@Component({
  selector: 'ngx-solution-comments',
  templateUrl: './solution-comments.component.html',
  styleUrls: ['./solution-comments.component.scss']
})
export class SolutionCommentsComponent extends BaseComponent implements OnInit {
  @Input() solution: Solution;
  commentForm: UntypedFormGroup;
  principal: SubatoPrincipal;

  constructor(
    private authService: AuthService,
    private commentService: SolutionCommentService,
    private notificationService: NotificationService,
    private fb: UntypedFormBuilder
  ) { 
    super();
  }

  ngOnInit(): void {
    this.createCommentForm();

    this.authService.principal$
    .pipe(
      takeUntil(this.unsubscribe)
    ).subscribe(principal => {
      this.principal = principal;
    })
  }

  createComment() {
    let dto: CreateSolutionCommentDto = {
      text: this.commentForm.get('text').value,
      solutionId: this.solution.id
    }

    this.commentService.create(dto)
      .pipe(takeUntil(this.unsubscribe))
      .subscribe(comment => {
        this.solution.comments.splice(0, 0, comment);
        this.notificationService.showSuccess("Kommentar erfolgreich erstellt");
        this.createCommentForm();
      }, (error: any) => {
        if (BackendError.isBackendError(error)) {
          this.notificationService.showError(
            error.format(
              "Fehler bei Erstellung des Kommentars" 
            )
          );
        } else {
          throw error;
        }
      });
  }

  deleteComment(comment: SolutionComment) {
    this.commentService.delete(comment.id)
      .pipe(takeUntil(this.unsubscribe))
      .subscribe(() => {
        var idx = this.solution.comments.map(c => c.id).indexOf(comment.id);
        this.solution.comments.splice(idx, 1);
        this.notificationService.showSuccess("Kommentar erfolgreich gelöscht");
      }, (error: any) => {
        if (BackendError.isBackendError(error)) {
          this.notificationService.showError(
            error.format(
              "Fehler bei Löschung des Kommentars" 
            )
          );
        } else {
          throw error;
        }
      });
  }

  createCommentForm() {
    this.commentForm = this.fb.group({
      text: ['', [Validators.required]],
    });
  }
}
