import { Component, Input, OnInit, ViewChild } from '@angular/core';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { FileTree } from 'app/shared/models/file-tree';
import { take, takeUntil } from 'rxjs/operators';
import { NotificationService } from 'app/shared/services/notification.service';
import { BackendError } from 'app/shared/models/backend-error';
import { Solution } from 'app/features/solutions/models/solution';
import { SolutionService } from 'app/features/solutions/services/solution.service';
import { ViewerHostDirective } from '../viewer/viewer-host.directive';
import { ViewerFactory } from '../viewer/viewer.factory';
import { Viewer } from '../viewer/viewer';

@Component({
  selector: 'solution-ide',
  templateUrl: './solution-ide.component.html',
  styleUrls: ['./solution-ide.component.scss']
})
export class SolutionIdeComponent extends BaseComponent implements OnInit {
  @Input() withBorder: boolean = false;

  loading: boolean;
  currentTree: FileTree;
  _solution: Solution;
  _compactMode: boolean = false;
  fileBrowserEnabled: boolean = false;

  Viewer = Viewer;
  currentViewer: Viewer = Viewer.none;

  defaultBrowserWidth: number = 15;
  browserWidth: number = this.defaultBrowserWidth;
  loadingViewer: boolean = false;
  viewerError: any;

  @ViewChild(ViewerHostDirective, { static: true }) viewerHost!: ViewerHostDirective;

  constructor(
    private solutionService: SolutionService,
    private notificationService: NotificationService,
    private viewerFactory: ViewerFactory
  ) {
    super();
  }

  ngOnInit(): void {
  }

  @Input() set solution(val: Solution) {
    this._solution = val;
    if (val.tree.children.length != 0) {
      let first = val.tree.children[0]
      this.compactMode = val.tree.children.length == 1 && first.children.length == 0;
      this.currentTree = this.compactMode ? first : null;
      this.fileBrowserEnabled = !this.compactMode;
    }

    this.createViewer();
  }

  get solution() {
    return this._solution;
  }

  get compactMode(): boolean {
    return this._compactMode;
  }

  set compactMode(val: boolean) {
    this._compactMode = val;
    this.browserWidth = this.compactMode ? 0 : this.defaultBrowserWidth;
  }

  createViewer(): void {
    const viewContainerRef = this.viewerHost.viewContainerRef;
    viewContainerRef.clear();
    this.loadingViewer = true;

    this.viewerFactory.createViewer(this.solution, this.currentTree, viewContainerRef)
      .pipe(
        take(1)
      )
      .subscribe(viewer => {
        this.currentViewer = viewer;
        this.loadingViewer = false;
      }, error => {
        this.viewerError = error;
        this.loadingViewer = false;
      });
  }

  onTreeSelected(tree: FileTree) {
    if (tree.children.length != 0) {
      return;
    }

    this.currentTree = tree;
    this.createViewer();
  }

  downloadSolution() {
    this.solutionService.downloadSolution(this.solution.id)
      .pipe(takeUntil(this.unsubscribe))
      .subscribe(() => { }, (error: any) => {
        if (BackendError.isBackendError(error)) {
          this.notificationService.showError(
            error.format(
              "Fehler beim Herunterladen der Lösung",
            )
          );
        } else {
          throw error;
        }
      });
  }

  toggleMenu() {
    this.compactMode = !this.compactMode;
  }
}
