import { Pipe, PipeTransform } from "@angular/core";
import { AppStateService } from "app/core/services/app.state.service";
import * as CodeMirror from "codemirror";
import { map } from "rxjs/operators";
import { CodemirrorThemePipe } from "./codemirror-theme.pipe";


/**
 * transforms all <pre> elements of the input html into codemirror elements, 
 * so that syntax highlighting is possible.
 * 
 * each <pre> element needs to have a 'language-<lang>' set, indicating how to highlight
 * the languages need to be mapped to codemirror modes using the langMap.
 * 
 * if no class is present, then the input html is returned as is.
 */
@Pipe({
    name: 'codemirrorHighlight',
})
export class CodemirrorHighlightPipe implements PipeTransform {
    // todo: rest endpoint for mapping language id -> mime type?
    langMap = {
        'java': 'text/x-java',
        'python': 'text/x-python',
        'py': 'text/x-python',
        'go': 'text/x-go',
        'kotlin': 'text/x-kotlin',
        'scala': 'text/x-scala',
        'haskell': 'text/x-haskell',
        'hs': 'text/x-haskell',
        'c': 'text/x-csrc',
        'js': 'text/javascript',
        'javascript': 'text/javascript',
        'cpp': 'text/x-c++src'
    };

    constructor(private appStateService: AppStateService) {
    }

    transform(inHtml: string): any {
        return this.appStateService.darkMode$
            .pipe(
                map(darkMode => {
                    return this.transformHtml(inHtml, darkMode);
                })
            );
    }

    private transformHtml(inHtml: string, darkMode: boolean) {
        let parser = new DOMParser();
        let doc = parser.parseFromString(inHtml, 'text/html');
        var preElements = doc.querySelectorAll("pre");

        preElements.forEach(pre => {
            // highlight content dependent on dom nodes, e.g.:
            // if <pre><code>...</code></pre> -> <code>...</code>
            // if <pre>...</pre> -> <pre>...</pre>
            let codeElement = (<any>pre.children[0]) ?? pre;

            if (codeElement) {
                let clazz = pre.attributes.getNamedItem('class')?.value;

                if (clazz && clazz.startsWith('language-')) {
                    let lang = clazz.split('-')[1];
                    if (lang) {
                        lang = lang.endsWith(';') ? lang.substring(0, lang.length - 1) : lang;
                        let mode = this.langMap[lang];

                        let outHtml = "";
                        console.log(mode, clazz);
                        CodeMirror.runMode(codeElement.innerText, mode, (text, clazz) => outHtml += `<span class="cm-${clazz}">${text}</span>`);
                        codeElement.innerHTML = outHtml;
                    }
                }

                let theme = new CodemirrorThemePipe().transform(darkMode);
                pre.setAttribute('class', `cm-s-${theme}`);
            }
        });

        return doc.body.innerHTML;
    }
}