package de.hsrm.sls.subato.auth.policy.solutions;

import de.hsrm.sls.subato.auth.Role;
import de.hsrm.sls.subato.auth.policy.SubatoPrincipalSpecifications;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyRule;
import de.hsrm.sls.subato.courses.course.Course_;
import de.hsrm.sls.subato.exercises.exercise.Exercise_;
import de.hsrm.sls.subato.solutions.solution.Solution;
import de.hsrm.sls.subato.solutions.solution.Solution_;
import de.hsrm.sls.subato.user.User_;
import jakarta.persistence.criteria.JoinType;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Component;


/**
 * The current user is allowed to view a solution
 */
@Component
public class SolutionViewPolicy implements PolicyRule {
    @Override
    public boolean check(CheckPolicyRequest request) {
        if(!request.matches(SolutionsAction.SOLUTION_VIEW)) {
            return false;
        }

        var solution = (Solution) request.getObj();

        var isAnonymousSolution = solution.getSubmittedBy() == null;
        var accessesOwnSolution = request.authenticated() && solution.getSubmittedBy().getId() == request.getSubject().getId();
        var principalIsCourseStaff = request.authenticated() && solution.getExercise() != null && request.getSubject().hasAnyGroup(solution.getExercise().getCourse().getId());
        var principalIsStaff = request.authenticated() && solution.getExercise() == null && request.getSubject().hasRole(Role.Staff);

        return isAnonymousSolution || accessesOwnSolution || principalIsCourseStaff || principalIsStaff;
    }

    @Override
    public Specification checkAsSpec(CheckPolicyRequest request) {
        if(!request.matches(SolutionsAction.SOLUTION_VIEW)) {
            return Specification.where(null);
        }

        return accessesOwnSolution(request.getSubject())
                .or(principalIsCourseStaff(request.getSubject()))
                .or(principalIsStaff(request.getSubject()));
    }

    public static Specification<Solution> accessesOwnSolution(SubatoPrincipal principal) {
        return (ex, cq, cb) -> cb.equal(ex.get(Solution_.submittedBy).get(User_.id), principal != null ? principal.getId() : -1);
    }

    public static Specification<Solution> principalIsCourseStaff(SubatoPrincipal principal) {
        return (ex, cq, cb) -> {
            var exerciseCourseJoin = ex
                    .join(Solution_.exercise, JoinType.LEFT)
                    .join(Exercise_.course, JoinType.LEFT);

            var courseStaffInBuilder = SubatoPrincipalSpecifications.courseStaffIn(principal, cb, exerciseCourseJoin.get(Course_.id));

            return cb.and(
                    cb.isNotNull(ex.get(Solution_.exercise)),
                    courseStaffInBuilder
            );
        };
    }

    public static Specification<Solution> principalIsStaff(SubatoPrincipal principal) {
        return (ex, cq, cb) -> cb.and(
                cb.isNull(ex.get(Solution_.exercise)),
                cb.equal(cb.literal(true), principal != null && principal.hasRole(Role.Staff))
        );
    }
}
