package de.hsrm.sls.subato.tasks.lang;

import jakarta.validation.ConstraintViolationException;
import jakarta.validation.Validation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Comparator;
import java.util.List;

@Service
public class LanguageService {
    @Autowired
    private LanguageRepository languageRepository;

    public Language findById(String id) {
        return languageRepository.findById(id).orElseThrow(() -> new LanguageNotFoundException(id));
    }

    public List<Language> findAll() {
        var languages = languageRepository.findAll();
        languages.sort(Comparator.comparing(Language::getId));
        return languages;
    }

    private void updateLanguage(Language language, CreateLanguageDto dto) {
        language.setId(dto.getId());
        language.setName(dto.getName());
    }

    /**
     * @throws ConstraintViolationException if invalid
     */
    private void validateLanguage(Language language) {
        var validator = Validation.buildDefaultValidatorFactory().getValidator();
        var violations = validator.validate(language);
        if (!violations.isEmpty()) {
            throw new ConstraintViolationException(violations);
        }
    }

    /**
     * @param dto attributes for the new language
     * @return created language
     * @throws ConstraintViolationException
     */
    public Language create(CreateLanguageDto dto) {
        var language = new Language();
        updateLanguage(language, dto);
        validateLanguage(language);

        return languageRepository.save(language);
    }

    /**
     * @throws ConstraintViolationException
     * @throws LanguageNotFoundException
     */
    public Language update(String id, CreateLanguageDto dto) {
        var language = findById(id);
        updateLanguage(language, dto);
        validateLanguage(language);

        return languageRepository.save(language);
    }

    /**
     * @throws LanguageNotFoundException
     */
    public void delete(String id) {
        var language = findById(id);
        languageRepository.delete(language);
    }
}
