package de.hsrm.sls.subato.tasks.task;

import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.tasks.TasksAction;
import de.hsrm.sls.subato.shared.FileResponseBuilder;
import de.hsrm.sls.subato.shared.helper.ResourceHelper;
import de.hsrm.sls.subato.shared.helper.ZipHelper;
import de.hsrm.sls.subato.tasks.feedback.*;
import de.hsrm.sls.subato.tasks.pool.TaskPoolReadyOnlyException;
import de.hsrm.sls.subato.tasks.stef.dto.FileRefDto;
import de.hsrm.sls.subato.tasks.stef.dto.FileRefMapper;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.Resource;
import org.springframework.core.io.UrlResource;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.web.PageableDefault;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import java.io.IOException;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import static org.springframework.http.HttpStatus.BAD_REQUEST;
import static org.springframework.http.HttpStatus.NOT_FOUND;

@RestController
@Tag(name = "Aufgabenverwaltung")
public class TaskController {
    @Autowired
    private TaskMapper taskMapper;

    @Autowired
    private FileRefMapper fileRefMapper;

    @Autowired
    private TaskFeedbackMapper taskFeedbackMapper;

    @Autowired
    private TaskService taskService;

    @Autowired
    private TaskFeedbackService taskFeedbackService;

    @Autowired
    private PolicyEvaluator policy;

    @Autowired
    private FileResponseBuilder fileResponseBuilder;

    @Autowired
    private TaskReferenceRewriter taskReferenceRewriter;

    @GetMapping("/task")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_VIEW)")
    @Operation(summary = "Liefert alle Aufgaben über alle Aufgaben-Pools hinweg")
    Page<TaskDto> getTasks(
            @PageableDefault Pageable page,
            @Parameter(description = "Case-insensitive Filter über Name und Beschreibung") @RequestParam("q") Optional<String> query,
            @Parameter(description = "Aufgaben-Pool Filter") @RequestParam("poolId") Optional<Long> poolId
    ) {
        var filter = new TaskFilter().query(query.orElse(null)).poolId(poolId.orElse(null));
        var tasks = taskService.getAll(page, filter);
        return tasks.map(task -> taskMapper.map(task));
    }

    @GetMapping("/task/{id}")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_VIEW)")
    @ApiResponses(value = {
        @ApiResponse(responseCode = "404", description = TaskNotFoundException.REASON),
    })
    @Operation(
            summary = "Liefert eine Aufgabe, wobei die relativen Verweise auf Bilder und andere Medien im Inhalt vorher aufgelöst werden",
            description = "Die Verweise werden so aufgelöst, sodass mit dem Endpunkt /task/{id}/assets/** die Inhalte direkt per HTTP eingebunden werden können"
    )
    TaskDto getOne(@PathVariable Long id) {
        try {
            var task = taskService.findById(id);
            var dto = taskMapper.map(task);
            return dto;
        } catch(TaskNotFoundException ex) {
            throw new ResponseStatusException(BAD_REQUEST, TaskNotFoundException.REASON);
        }
    }

    @GetMapping("/task/{id}/content")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_VIEW)")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "404", description = TaskNotFoundException.REASON),
            @ApiResponse(responseCode = "400", description = TaskArchivedException.REASON),
    })
    String getContent(@PathVariable Long id, HttpServletRequest req) {
        try {
            var task = taskService.findById(id);
            return taskReferenceRewriter.rewrite(task, req);
        } catch(TaskNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, TaskNotFoundException.REASON);
        } catch(TaskArchivedException ex) {
            throw new ResponseStatusException(BAD_REQUEST, TaskArchivedException.REASON);
        }
    }

    @GetMapping("/task/{id}/files")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_VIEW)")
    @Operation(summary = "Alle Dateien zur Aufgabe")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "404", description = TaskNotFoundException.REASON),
            @ApiResponse(responseCode = "400", description = TaskArchivedException.REASON),
    })
    List<FileRefDto> getFiles(@PathVariable Long id) {
        try {
            var task = taskService.findById(id);
            var files = taskService.getFiles(task);
            return files.stream().map(file -> fileRefMapper.map(file)).collect(Collectors.toList());
        } catch(TaskNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, TaskNotFoundException.REASON);
        } catch(TaskArchivedException ex) {
            throw new ResponseStatusException(BAD_REQUEST, TaskArchivedException.REASON);
        }
    }

    @GetMapping("/task/{id}/files/{uuid}/download")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_VIEW)")
    @Operation(summary = "Zugriff auf den Inhalt der Datei zu einer Aufgabe")
    ResponseEntity<Resource> downloadFile(@PathVariable Long id, @PathVariable String uuid) throws IOException {
        try {
            var task = taskService.findById(id);
            var file = taskService.getFile(task, uuid);

            if (file.isDirectory()) {
                var zipFile = ZipHelper.zip(file, true);
                var resource = new UrlResource(zipFile.toURI());
                return fileResponseBuilder.build(resource, "%s.zip".formatted(file.getName()), true);
            } else {
                var resource = ResourceHelper.load(file);
                return fileResponseBuilder.build(resource, true);
            }
        } catch(FileNotFoundException e) {
            throw new ResponseStatusException(NOT_FOUND, FileNotFoundException.REASON);
        } catch(TaskArchivedException ex) {
            throw new ResponseStatusException(BAD_REQUEST, TaskArchivedException.REASON);
        }
    }

    @GetMapping("/task/{id}/spec")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_VIEW)")
    @Operation(summary = "Übermittlung der STEF-Spezifikation für die Aufgabe als zip-Datei")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "404", description = TaskNotFoundException.REASON),
    })
    ResponseEntity<Resource> downloadSpec(@PathVariable Long id) throws IOException {
        try {
            var task = taskService.findById(id);
            var zipFile = taskService.getSpecAsZip(task);
            var resource = new UrlResource(zipFile.toURI());
            return fileResponseBuilder.build(resource, "%s.zip".formatted(task.getIdName()), true);
        } catch(TaskNotFoundException ex) {
            throw new ResponseStatusException(BAD_REQUEST, TaskNotFoundException.REASON);
        }
    }

    @GetMapping("/task/{id}/asset")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_VIEW)")
    @Operation(
            summary = "Zugriff auf eine Datei im Text-Ordner in der STEF Spezifikation durch Angabe eines relativen Pfades",
            description = "Ein Aufruf von ```/task/15/assets/images/BTree.png``` würde z.B. auf ```/15SomeTask/text/images/BTree.png``` verweisen"
    )
    @ApiResponses(value = {
            @ApiResponse(responseCode = "404", description = TaskNotFoundException.REASON + " oder " + AssetNotFoundException.REASON),
    })
    ResponseEntity<Resource> downloadAsset(@PathVariable Long id, @RequestParam("path") String path) throws IOException {
        try {
            var task = taskService.findById(id);
            var file = taskService.getTextAsset(task, path);

            var resource = ResourceHelper.load(file);
            return fileResponseBuilder.build(resource, file.getName(), true);
        } catch(TaskNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, TaskNotFoundException.REASON);
        } catch(AssetNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, "%s: %s".formatted(AssetNotFoundException.REASON, ex.getPath(), id));
        }
    }

    @GetMapping("/task/{id}/feedback")
    @Operation(summary = "Liefert alle Feedbackeinträge von Benutzern zu dieser Aufgabe")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "404", description = TaskNotFoundException.REASON),
    })
    List<TaskFeedbackDto> getFeedback(@PathVariable Long id) {
        try {
            var feedback = taskFeedbackService.getPendingFeedbackForTask(id);
            feedback = feedback.stream()
                    .filter(fb -> policy.check(CheckPolicyRequest.fromPrincipal(fb, TasksAction.TASK_FEEDBACK_VIEW)))
                    .toList();
            return feedback.stream().map(f -> taskFeedbackMapper.map(f)).collect(Collectors.toList());
        } catch(TaskNotFoundException ex) {
            throw new ResponseStatusException(BAD_REQUEST, TaskNotFoundException.REASON);
        }
    }

    @GetMapping("/task/{taskId}/feedback/{userId}")
    @Operation(summary = "Liefert den Feedbackeintrag eines Benutzers zu dieser Aufgabe")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "404", description = TaskFeedbackNotFoundException.REASON),
    })
    TaskFeedbackDto getFeedback(@PathVariable Long taskId, @PathVariable Long userId) {
        try {
            var feedback = taskFeedbackService.getFeedbackForTaskByUser(taskId, userId);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(feedback, TasksAction.TASK_FEEDBACK_VIEW));

            return feedback != null ? taskFeedbackMapper.map(feedback) : null;
        } catch(TaskFeedbackNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, "%s (Aufgabe %s, Benutzer %s)".formatted(TaskFeedbackNotFoundException.REASON, taskId, userId));
        }
    }

    @PutMapping("/task/{taskId}/feedback")
    @Operation(summary = "Erstellung eines Feedbackeintrag zu dieser Aufgabe. Aktueller Benutzer ist Autor des Feedbacks")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "404", description = TaskNotFoundException.REASON),
    })
    TaskFeedbackDto createFeedback(@PathVariable Long taskId, @RequestBody CreateTaskFeedbackDto dto) {
        try {
            var feedback = taskFeedbackService.createOrUpdate(dto, taskId);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(feedback, TasksAction.TASK_FEEDBACK_UPDATE));

            return taskFeedbackMapper.map(feedback);
        } catch(TaskNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, TaskNotFoundException.REASON);
        }
    }

    @PutMapping("/task/{id}/feedback/{userId}/read")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_FEEDBACK_UPDATE)")
    @Operation(summary = "Markierung eines Feedbackeintrags als gelesen.")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "404", description = TaskFeedbackNotFoundException.REASON),
    })
    TaskFeedbackDto markFeedbackAsRead(@PathVariable Long id, @PathVariable Long userId) {
        try {
            var feedback = taskFeedbackService.markAsRead(id, userId);
            return taskFeedbackMapper.map(feedback);
        } catch(TaskFeedbackNotFoundException ex) {
            throw new ResponseStatusException(BAD_REQUEST, TaskFeedbackNotFoundException.REASON);
        }
    }

    @DeleteMapping("/task/{id}")
    @PreAuthorize("hasPermission(null, @TasksAction.TASK_ARCHIVE)")
    @Operation(summary = "Archiviert eine Aufgabe, sodass sie nicht mehr verfügbar ist aber bestehende Links (von z.B. Lösungen) bestehen bleiben")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "400", description = TaskPoolReadyOnlyException.REASON),
            @ApiResponse(responseCode = "404", description = TaskNotFoundException.REASON),
    })
    void delete(@PathVariable Long id) {
        try {
            var task = taskService.findById(id);
            if(task.getPool().isLinkedToRepository()) {
                throw new TaskPoolReadyOnlyException();
            }

            taskService.archive(task);
        } catch(TaskNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, TaskNotFoundException.REASON);
        } catch(TaskPoolReadyOnlyException ex) {
            throw new ResponseStatusException(BAD_REQUEST, TaskPoolReadyOnlyException.REASON);
        }
    }
}
